@@ -15,7 +15,7 @@
 #
 
 package MongoDB::BSON;
-our $VERSION = '0.35';
+our $VERSION = '0.36';
 
 # ABSTRACT: Tools for serializing and deserializing data in BSON form
 use Any::Moose;
@@ -57,4 +57,16 @@ If you turn off utf8 flag, the string  length will compute as bytes, and is_utf8
 
 =cut
 
-$MongoDB::BSON::utf8_flag_on = 1;
\ No newline at end of file
+$MongoDB::BSON::utf8_flag_on = 1;
+
+=head2 Return boolean values as booleans instead of integers
+
+    $MongoDB::BSON::use_boolean = 1
+
+By default, booleans are deserialized as integers.  If you would like them to be
+deserialized as L<boolean/true> and L<boolean/false>, set 
+C<$MongoDB::BSON::use_boolean> to 1.
+
+=cut
+
+$MongoDB::BSON::use_boolean = 0;
